import os
import sys
import time

from gettext import gettext as _

def get_lpid():
    if not os.path.exists(os.path.expanduser("~/.5-a-day")):
        print >> sys.stderr, _("""Please type in:
        echo <Your Launchpad ID> > ~/.5-a-day""")
        return ""

    return open(os.path.expanduser("~/.5-a-day")).read().strip().lower()
    
def signature_file():
    return os.path.expanduser("~/.signature")
    
def old_signature_file():
    return os.path.expanduser("~/.signature.old")

def team_file():
    return os.path.expanduser("~/.5-a-day-team")

def team_overview_file():
    return os.path.expanduser("~/.5-a-day-data/.teams")

def tag_file():
    return os.path.expanduser("~/.5-a-day-tags")

def get_tags():
    if os.path.exists(tag_file()):
        return map(lambda a: a.strip(), open(tag_file()).readlines())
    return []

def local_branch():
    return os.path.expanduser("~/.5-a-day-data")

def log_file():
    return os.path.expanduser("~/.5-a-day-data/%s" % get_lpid())

def preserve_old_signature():
    sig = signature_file()
    oldsig = old_signature_file()
    if os.path.exists(sig):
        if not open(sig).read().count("https://wiki.ubuntu.com/5-A-Day") and \
           not os.path.exists(oldsig):
            os.system("mv %s %s" % (sig, oldsig))
        else:
            os.remove(sig)

def get_old_log():
    if os.path.exists(log_file()):
        return open(log_file()).readlines()
    else:
        return ""

class TeamFile(object):
    def __init__(self):
        self.teams = []
        if os.path.exists(team_file()):
            self.teams = map(lambda a: a.strip(), 
                             open(team_file()).readlines())

class TeamOverviewFile(object):
    def __init__(self):
        self.overview = {}
        if os.path.exists(team_overview_file()):
            for l in open(team_overview_file()).readlines():
                args = l.split()
                if len(args) > 1:
                    self.overview[args[0]] = args[1:]
    def add(name, teams):
        f = open(teamoverviewfile, "a")
        f.write("%s %s\n" % (name, " ".join(teams)))
        f.close()

def first_element_is_int(s):
    a = s.split()[0]
    try:
        i = int(a)
        return True
    except:
        return False

class LogFile(object):
    def __init__(self):
        self.tags = get_tags()
        if os.path.exists(log_file()):
            self.entries = map(lambda a: a.strip(),
                               open(log_file()).readlines())
            # we used "2008-4-1 123456" entries before, now they're simply
            # <timestamp> <bug>
            self.correct_new_entries = filter(lambda a: first_element_is_int(a),
                                              self.entries)
        else:
            self.entries = []
            self.correct_new_entries = []

    def add(self, t, entries):
        # TODO: move parsing/validation of bug numbers to parser?!
        #       This method should only get a list of bug numbers probably.

        # Parse/validate bug arguments:
        bugs = []
        for entry in entries:
            for i in entry.split(","):
                n = i.strip(" #")
                if n:
                    try:
                        number = int(n)
                    except:
                        print >> sys.stderr, _("'%s' is not a valid bug number. Aborting.") % n
                        return 105 #105: invalid bug number
                    if filter(lambda a: (int(t)-int(a.split()[0]) < 60*60*24) and \
                                        (a.split()[1]==str(n)), 
                              self.correct_new_entries):
                        print >> sys.stderr, _("Bug '%s' has already been added today. Aborting.") % n
                        return 106 #106: bug has already been added today
                    bugs.append(n)
        # Write bugs to file:
        if os.path.exists(log_file()):
            f = open(log_file(), "a")
        else:
            f = open(log_file(), "w")
        for bug in bugs:
            data_str = "%s %s %s\n" % (int(t), bug, " ".join(self.tags))
            f.write(data_str)
        f.close()

def modify_tags(args):
    tags = get_tags()
    try:
        os.remove(tag_file())
    except:
        pass

    if args[0] == "--remove-tag" and args[1] in tags:
        tags.remove(args[1])
    if args[0] == "--add-tag" and args[1] not in tags:
        tags.append(args[1])

    f = open(tag_file(), "w")
    f.write("\n".join(tags))
    f.close()

    
